
import React, { useEffect, useMemo } from "react";
import { createBrowserRouter, RouterProvider } from "react-router-dom";
import { QueryClient, QueryClientProvider } from '@tanstack/react-query';
import ThemeProvider from "@/components/ThemeProvider";
import { Toaster } from "sonner";
import { PerformanceMonitoringService } from "@/lib/performanceMonitoring";
import { CriticalResourcePreloader } from "@/lib/bundleOptimization";
import { LazyImageLoader } from "@/lib/imageOptimization";

// Optimized lazy imports with error boundaries
import {
  LazyIndex,
  LazySearch,
  LazyCategories,
  LazyCategoryView,
  LazyEmojiDetailEnhanced,
  LazyTrending,
  LazyTrendingPhrases,
  LazyEmojiPhrases,
  LazyEmojiPhraseDetail,
  LazyRandom,
  LazyGenerator,
  LazyTranslator,
  LazyEmojiAdmin,
  LazySeoAdmin,
  LazyNotFound,
  withLazyLoading,
  LazyLoadErrorBoundary
} from "@/utils/routeSplitting";

// Optimized router configuration with prefetching
const router = createBrowserRouter([
  {
    path: "/",
    element: withLazyLoading(LazyIndex, "/")({})
  },
  {
    path: "/search",
    element: withLazyLoading(LazySearch, "/search")({})
  },
  {
    path: "/categories",
    element: withLazyLoading(LazyCategories, "/categories")({})
  },
  {
    path: "/categories/:categorySlug",
    element: withLazyLoading(LazyCategoryView, "/categories")({})
  },
  {
    path: "/emoji/:slug",
    element: withLazyLoading(LazyEmojiDetailEnhanced, "/emoji")({})
  },
  {
    path: "/trending",
    element: withLazyLoading(LazyTrending, "/trending")({})
  },
  {
    path: "/trending-phrases",
    element: withLazyLoading(LazyTrendingPhrases, "/trending-phrases")({})
  },
  {
    path: "/emoji-phrases",
    element: withLazyLoading(LazyEmojiPhrases, "/emoji-phrases")({})
  },
  {
    path: "/emoji-phrases/:categorySlug",
    element: withLazyLoading(LazyEmojiPhraseDetail, "/emoji-phrases")({})
  },
  {
    path: "/random",
    element: withLazyLoading(LazyRandom, "/random")({})
  },
  {
    path: "/generator",
    element: withLazyLoading(LazyGenerator, "/generator")({})
  },
  {
    path: "/translator",
    element: withLazyLoading(LazyTranslator, "/translator")({})
  },
  {
    path: "/admin",
    element: withLazyLoading(LazyEmojiAdmin, "/admin")({})
  },
  {
    path: "/admin/seo",
    element: withLazyLoading(LazySeoAdmin, "/admin/seo")({})
  },
  {
    path: "*",
    element: withLazyLoading(LazyNotFound, "/404")({})
  }
]);

function App() {
  // Create QueryClient inside the component to ensure React context is available
  const queryClient = useMemo(() => new QueryClient({
    defaultOptions: {
      queries: {
        staleTime: 30 * 60 * 1000, // 30 minutes - increased for better caching
        gcTime: 2 * 60 * 60 * 1000, // 2 hours - better memory management
        retry: (failureCount, error: any) => {
          // Smart retry logic
          if (error?.status === 404) return false;
          return failureCount < 2;
        },
        refetchOnWindowFocus: false,
        refetchOnMount: false,
        refetchOnReconnect: false,
        // Enable background updates for better UX
        refetchInterval: 5 * 60 * 1000, // 5 minutes for trending data
      },
      mutations: {
        retry: 1,
      },
    },
  }), []);

  useEffect(() => {
    // Initialize performance monitoring
    PerformanceMonitoringService.init();
    
    // Preload critical resources
    CriticalResourcePreloader.preloadCriticalImages();
    
    // Initialize lazy image loading
    LazyImageLoader.init();
    
    // Cleanup on unmount
    return () => {
      LazyImageLoader.disconnect();
    };
  }, []);

  return (
    <LazyLoadErrorBoundary>
      <QueryClientProvider client={queryClient}>
        <ThemeProvider defaultTheme="system" storageKey="emoji-crush-theme">
          <Toaster 
            position="top-right" 
            toastOptions={{
              duration: 3000,
              style: {
                background: 'hsl(var(--background))',
                color: 'hsl(var(--foreground))',
                border: '1px solid hsl(var(--border))',
              },
            }}
          />
          <RouterProvider router={router} />
        </ThemeProvider>
      </QueryClientProvider>
    </LazyLoadErrorBoundary>
  );
}

export default App;
